@@ -19,7 +19,7 @@
 #' the "second" spans the last three columns, "a" spans the first two, "b"
 #' the middle column, and "c" the last two columns.
 #'
-#' @section The \code{rgroup} arguement:
+#' @section The \code{rgroup} argument:
 #'
 #'  The rgroup allows you to smoothly group rows. Each row within a group
 #'  receives an indention of two blank spaces and are grouped with their
@@ -80,6 +80,25 @@
 #' for compatibility reasons. If you set \code{options(table_counter_roman = TRUE)}
 #' then the table counter will use Roman numumerals instead of Arabic.
 #'
+#'@section The \code{css.cell} argument:
+#'
+#' The \code{css.cell} parameter allows you to add any possible CSS style
+#' to your table cells.  \code{css.cell} can be either a vector or a matrix.
+#'
+#' If  \code{css.cell} is a \emph{vector}, it's assumed that the styles should be repeated
+#' throughout the columns (that is, each element in css.cell specify the style
+#' for the whole row of 'x').
+#'
+#' In the case of  \code{css.cell} being a \emph{matrix} of the same size of the \code{x} argument,
+#' each element of \code{x} gets the style from the corresponding element in css.cell.  Additionally,
+#' the number of rows of \code{css.cell} can be \code{nrow(x) + 1} so the first row of of \code{css.cell}
+#' specifies the style for the header of \code{x}; also the number of columns of \code{css.cell}
+#' can be \code{ncol(x) + 1} to include the specification of style for row names of \code{x}.
+#'
+#'@section Empty dataframes:
+#' An empty dataframe will result in a warning and output an empty table, provided that
+#' rgroup and n.rgroup are not specified. All other row layout options will be ignored.
+#'
 #' @section Browsers and possible issues:
 #'
 #' \emph{Copy-pasting:} As you copy-paste results into Word you need to keep
@@ -187,11 +206,7 @@
 #' @param css.tspanner.sep The line between different spanners
 #' @param css.total The css of the total row
 #' @param css.cell The css.cell element allows you to add any possible CSS style to your
-#'  table cells. If you provide a vector the vector it is assummed that the styles should
-#'  be repeated throughout the columns. If you provide a matrix of the same size as your
-#'  \code{x} argument. If have \code{ncol(x) + 1} the first row will correspond to the
-#'  rowname style. Correspondingly if the size is \code{nrow(x) + 1} it is assummed that the
-#'  first row is the header row.
+#'  table cells. See section below for details.
 #' @param css.class The html CSS class for the table. This allows directing html
 #'  formatting through \href{http://www.w3schools.com/Css/}{CSS}
 #'  directly at all instances of that class. \emph{Note:} unfortunately the
@@ -251,6 +266,7 @@ htmlTable <- function(x, ...){
 
 #' @importFrom stringr str_trim
 #' @importFrom stringr str_replace
+#' @import checkmate
 #' @import magrittr
 #' @rdname htmlTable
 #' @export
@@ -310,51 +326,25 @@ htmlTable.default <- function(x,
                               cspan.rgroup = "all",
                               ...)
 {
-  # Warnings due to interface changes in 1.0
-  API_changes <-
-    c(rowname = "rnames",
-      headings = "header",
-      halign = "align.header",
-      cgroup.just = "align.cgroup",
-      rgroupCSSstyle = "css.rgroup",
-      rgroupCSSseparator = "css.rgroup.sep",
-      tspannerCSSstyle = "css.tspanner",
-      tspannerCSSseparator = "css.tspanner.sep",
-      rgroup.padding = "padding.rgroup",
-      rowlabel.pos =  "pos.rowlabel",
-      caption.loc  = "pos.caption",
-      altcol = "col.rgroup",
-      tableCSSclass = "css.class")
-  dots <- list(...)
-  fenv <- environment()
-  for (i in 1:length(API_changes)){
-    old_name <- names(API_changes)[i]
-    new_name <- API_changes[i]
-    if (old_name %in% names(dots)){
-      if (class(fenv[[new_name]]) == "name"){
-        fenv[[new_name]] <- dots[[old_name]]
-        dots[[old_name]] <- NULL
-        warning("Deprecated: '", old_name, "'",
-                " argument is now '", new_name ,"'",
-                " as of ver. 1.0")
-      }else{
-        stop("You have set both the old parameter name: '", old_name, "'",
-             " and the new parameter name: '", new_name, "'.",
-             " Note that parameters may have a default value and you may have only",
-             " set the old paramter while the function automatically attaches a value to the new parameter")
-      }
-    }
-  }
-
   if (is.null(dim(x))){
+    if (!is.numeric(x) && !is.character(x)){
+      x <- as.character(x)
+    }
     x <- matrix(x, ncol = ifelse(missing(header),
                                  length(x),
                                  length(header)))
-  }else if (length(dim(x)) != 2)
+  }else if (length(dim(x)) != 2) {
     stop("Your table variable seems to have the wrong dimension,",
          " length(dim(x)) = ", length(dim(x)) , " != 2")
+  }
 
 
+  if (missing(rgroup) &&
+      !missing(n.rgroup)){
+    # Add "" rgroups corresponding to the n.rgroups
+    rgroup = rep("", length.out=length(n.rgroup))
+  }
+
   ## this will convert color names to hexadecimal (easier for user)
   ## but also leaves hex format unchanged
   col.rgroup <- prPrepareColors(col.rgroup, n = nrow(x), ng = n.rgroup, gtxt = rgroup)
@@ -365,16 +355,15 @@ htmlTable.default <- function(x,
   if (missing(rnames)){
     if (any(is.null(rownames(x)) == FALSE))
       rnames <- rownames(x)
-  }
 
-  if (missing(rnames) &&
-        any(is.null(rownames(x))) &&
+    if (any(is.null(rownames(x))) &&
         !missing(rgroup)){
-    warning("You have not specified rnames but you seem to have rgroups.",
-            " If you have the first column as rowname but you want the rgroups",
-            " to result in subhedings with indentation below then, ",
-            " you should change the rnames to the first column and then",
-            " remove it from the table matrix (the x argument object).")
+      warning("You have not specified rnames but you seem to have rgroups.",
+              " If you have the first column as rowname but you want the rgroups",
+              " to result in subhedings with indentation below then, ",
+              " you should change the rnames to the first column and then",
+              " remove it from the table matrix (the x argument object).")
+    }
   }
 
   if (!missing(rowlabel) &&
@@ -562,6 +551,8 @@ htmlTable.default <- function(x,
     table_id <- sprintf(" id='%s'", label)
   }else if(is.numeric(table_id)){
     table_id <- paste0(" id='table_", table_id, "'")
+  }else if(table_id == FALSE){
+    table_id <- ""
   }
 
   # A column counter that is used for <td colspan="">
@@ -612,6 +603,10 @@ htmlTable.default <- function(x,
 
   css.total <- rep(css.total, length.out = length(total))
 
+  assert(
+    check_matrix(css.cell),
+    check_character(css.cell)
+  )
   css.cell <- prPrepareCss(x, css = css.cell,
                            rnames = rnames, header = header)
 
@@ -627,9 +622,14 @@ htmlTable.default <- function(x,
   # import to word processors works then less well and therefore I've
   # constructed this work-around with borders for the top and bottom cells
   first_row <- TRUE;
-  if (ctable){
+  if (isTRUE(ctable)){
     top_row_style = "border-top: 2px solid grey;"
     bottom_row_style = "border-bottom: 2px solid grey;"
+  } else if (any(ctable %in% c('single', 'double'))) {
+    ctable <- rep_len(ctable, 2L)
+    ctable[ctable %in% 'single'] <- 'solid'
+    top_row_style = ifelse(ctable[1] ==  'solid', "border-top: 2px solid grey;", "border-top: 4px double grey;")
+    bottom_row_style = ifelse(ctable[2] ==  'solid', "border-bottom: 2px solid grey;", "border-bottom: 4px double grey;")
   } else {
     top_row_style = "border-top: 4px double grey;"
     bottom_row_style = "border-bottom: 1px solid grey;"
@@ -692,6 +692,7 @@ htmlTable.default <- function(x,
 
   rgroup_iterator <- 0
   tspanner_iterator <- 0
+  if(nrow(x) > 0){
   for (row_nr in 1:nrow(x)){
     rname_style = attr(css.cell, "rnames")[row_nr + !prSkipRownames(rnames)]
 
@@ -733,6 +734,7 @@ htmlTable.default <- function(x,
     # and it's:
     # - first row
     # - the row belongs to the next row group
+    rgroup_sep_style <- FALSE
     if (!missing(rgroup) &&
       (row_nr == 1 ||
         row_nr > sum(n.rgroup[1:rgroup_iterator]))){
@@ -753,7 +755,7 @@ htmlTable.default <- function(x,
 
       # Only add if there is anything in the group
       if (is.na(rgroup[rgroup_iterator]) == FALSE &&
-            rgroup[rgroup_iterator] != ""){
+          rgroup[rgroup_iterator] != ""){
 
         if (first_row){
           rs <- c(rs,
@@ -777,6 +779,9 @@ htmlTable.default <- function(x,
           paste(rgroup_str)
 
         first_row <- FALSE
+      }else if(rgroup_iterator > 1 && css.rgroup.sep[rgroup_iterator-1] != ""){
+        # Add the separator if the rgroup wasn't added so that it's included in the regular cells
+        rgroup_sep_style = css.rgroup.sep[rgroup_iterator-1]
       }
     }
 
@@ -787,6 +792,8 @@ htmlTable.default <- function(x,
         c(top_row_style)
       cell_style %<>%
         c(top_row_style)
+    }else if(rgroup_sep_style != FALSE){
+      rs %<>% c(rgroup_sep_style)
     }
     first_row <- FALSE
 
@@ -844,7 +851,7 @@ htmlTable.default <- function(x,
     table_str %<>%
       paste0(cell_str, "\n\t</tr>")
   }
-
+}
   # Close body
   table_str %<>%
     paste0("\n\t</tbody>")
@@ -898,18 +905,43 @@ prConvertDfFactors <- function(x){
   if (!"data.frame" %in% class(x))
     return(x)
 
-  i <- sapply(x, is.factor)
+  i <- sapply(x, function(col)
+    (
+      (
+        !is.numeric(col) &&
+          !is.character(col)
+      ) ||
+      (
+        inherits(col, "times") # For handlin Chron input
+      )
+    )
+  )
+
   if(any(i)){
     x[i] <- lapply(x[i], as.character)
   }
+
   return (x)
 }
 
 #' @export
 htmlTable.data.frame <- function(x, ...) {
+  # deal gracefully with an empty dataframe - issue a warning.
+  if(nrow(x) == 0){
+    warning(paste(deparse(substitute(x)), "is an empty object"))
+  }
   htmlTable.default(prConvertDfFactors(x),...)
 }
 
+#' @export
+htmlTable.matrix <- function(x, ...) {
+  # deal gracefully with an empty dataframe - issue a warning.
+  if(nrow(x) == 0){
+    warning(paste(deparse(substitute(x)), "is an empty object"))
+  }
+  htmlTable.default(x,...)
+}
+
 #' @importFrom methods setClass
 setClass("htmlTable", contains = "character")
 
@@ -1074,3 +1106,4 @@ tblNoNext <- function(roman = getOption("table_counter_roman",
 
   return(next_no)
 }
+
